; QB Copyright (C)2001 Andrew Davie
; see http://homepages.ihug.co.nz/~aaronat/qb.html

; DO NOT TRUST ABSOLUTELY THE COMMENTS IN THIS CODE!
; MOST OF THEM ARE OK.  SOME ARE MISLEADING :)

; History: original PUSH demo posted to Stella list May 1998
; This demonstrated multiple-colour playfields and VERY rudimentary background animation

; Extended January/February 2001 to display a 4x4 tiled grid, each tile independantly movable.



    processor 6502
    include vcs.h

 
CUBEXY          = 5                     ; size of cube + border
PFSIZE          = 3*(4*CUBEXY-1)        ; 3 bytes / line by n lines
NPLAYERS        = 2                     ; DO NOT CHANGE

SPRITE_Y_ADJUST = 2                    ; adjust vertical position of sprites
SPRITE_X_ADJUST = 22                    ; horizontal pixel adjust to align with grid

GUARANTEED0READ = ZERO                  ; ANY zero page locations with 0,0 in consecutive bytes


NUMBEROFCUBES   = 7                     ; cubes which move
 ;\---> see dependencies :
 ;       - PlayerCube assumes <= 8
 ;       - uses 2n bytes RAM
 ;      NOT MUCH POINT >8 CUBES, AS IT GETS CLUTTERED/EASY!


SPRITE_HEIGHT   = 10

 
; bits
D0 = 1
D1 = 2
D2 = 4
D3 = 8
D4 = 16
D5 = 32
D6 = 64
D7 = 128

;------------------------------------------------------------------------------
; Timing adjustments
; These values control gameplay timing, and may be adjusted in the range 0-31

TIME_PAUSE_AFTER_JUMP   = 5

    
;------------------------------------------------------------------------------
; variables

    SEG.U vars
    org $80

    ; only 128 bytes... including the stack!
    ; let the assembler calculate the addresses

ZERO            ds 2                    ; ZERO! (2 bytes, as MTargetArea indexing is 0,y and 1,y)

PlayerCounter   ds NPLAYERS             ; generic counter for event timing
                ; mmmccccc
                ; m = creature mode
                ; c = creature counter
        ;\--> mode indicates what to do
        ;     counter is used for THAT mode only

        ;\--> usage/mode definitions
MODEMASK        = %11100000
MODE_NORMAL     = %00000000                     ; doing nothing special
MODE_RISE       = %00100000                     ; rising up out of cubes
MODE_SINK       = %01000000                     ; sinking into a cube
MODE_LURK       = %01100000                     ; lurking, waiting for a cube
MODE_JUMP       = %10000000                     ; jumping
MODE_LOCK       = %10100000                     ; locked, pending screen change
MODE_DELAY      = %11000000                     ; just delays

CubeX           ds NUMBEROFCUBES        ; d0-d4 = X(0-31), high 3 bits unused.
                ; xxx11111
                ; xxx unused

CubeYDir        ds NUMBEROFCUBES
                ; low nibble = Y (0-15) high nibble = direction
                ; direction nibble:
                ;       0x00 up
                ;       0x01 right
                ;       0x10 down
                ;       0x11 left
                ;       1x00 stationary
                ; x unused
                ; $FF indicates cube is NOT present

rnd             ds 2                    ; random #

PlayerFlags     ds NPLAYERS                     ; generic flags
                ; ddxxfffr
                ; d = direction of moving/facing. 00=up, 01=right, 10=down, 11=left
                ; f = frame to display for animation
                ; r = use creature counter as rise/fall display offset for sprites
                ; x = unused
        ;\--> usage definitions
FLAG_RISE       = D0                            ; use counter as rise/fall

PlayerX         ds NPLAYERS                     ; x coordinate
PlayerY         ds NPLAYERS                     ; y offset for jumping, etc.

PlayerCube      ds NPLAYERS
                ; values 0-NUMBEROFCUBES-1
                ; Let's assume NUMBEROFCUBES <= 8  (OK, so that's only 3 bits required)
                ; xddddiii
                ; i, d0-d2 = cube index
                ; d, d3-d6 = sprite/creature ID (used for graphic selection) (16 objects)
                ; x, d7 unused

TARGETSIZE = 5
TargetDisplay   ds 2*TARGETSIZE                 ; visible target display (flashes/blinks, etc)
                ; TargetDisplay hold the graphics for the entire right-side of the screen
                ; The first 4 bytes in each are the target area, which is flashed/animated
                ; The last byte is (all of!) the sliding cube feeding onto the grid.

Grid            ds 4                    ; 4 rows.  2 bits/tile for info
                ; GRID: organised into 4 rows, 1 byte/row
                ;       each byte has 2 bits/tile, ordered target/ownership
                ;       target = a cube 'belongs' here
                ;       ownership = a cube IS here

playfield       ds PFSIZE

level           ds 1                    ; game level (increases difficulty)
        ; mmmlllll
        ; mmm = # of lives (0-7)
        ; lllll = level (0-31)

Cubes           ds 1                    ; number of cubes on this board

BackColour      ds 1                    ; background colour (mainly debugging)

PlayerScore     ds 3                    ; player's score (3 BCD digits)

        ; /DODGY ON
        ; This overlay variable is used for the overlay variables.  That's OK.
        ; However, it is positioned at the END of the variables so, if on the off chance we're overlapping
        ; stack space and variable, it is LIKELY that that won't be a problem, as the temp variables
        ; (especially the latter ones) are only used in rare occasions.

        ; FOR SAFETY, DO NOT USE THIS AREA DIRECTLY (ie: NEVER reference 'Overlay' in the code)
        ; ADD AN OVERLAY FOR EACH ROUTINE'S USE, SO CLASHES CAN BE EASILY CHECKED

timer           ds 1                    ; generic 1-per-frame timer

Overlay         ds 15  ;->FF (in theory :)

        ; /DODGY OFF

;------------------------------------------------------------------------------
; OVERLAYS!
; These variables are overlays, and should be managed with care
; That is, variables are ALREADY DEFINED, and we're reusing RAM for other purposes

; EACH OF THESE ARE VARIABLES (TEMPORARY) USED BY ONE ROUTINE (AND IT'S SUBROUTINES)
; THAT IS, LOCAL VARIABLES.  USE 'EM FREELY, THEY COST NOTHING

; TOTAL SPACE USED BY ANY OVERLAY GROUP SHOULD BE <= SIZE OF 'Overlay'

;------------------------------------------------------------------------------
; MTarget(Area) overlays...
        
        org Overlay

MT_TargetMask   ds 1                    ; mask for target area border (SHARED)

; MTargetArea overlays... (SHARES SOME WITH MTARGETAREA)

MTA_TargetL     ds 1                    ; calculated target-area display bytes
MTA_TargetR     ds 1

SpriteLine      ds 2
Sprite          ds 4                    ; -> sprite graphic shape

; DrawScreen overlays...

DS_temp         ds 1
DS_HalfY        ds 1
DS_Sprite       ds 2

; SUBSETS of DrawScreen
; PositionSprites overlays...

PS_temp         ds 1

;------------------------------------------------------------------------------
; ScoreLine overlays...
; OVERLAP WITH DRAWSCREEN... BE *VERY* CAREFUL NOT TO BRACKET USAGE
; ACROSS SCORELINE!

        org Overlay

stkp            ds 1
sreg            ds 1
loop            ds 1

S0              ds 2                    ; used for addressing digits of score
S1              ds 2
S2              ds 2
S3              ds 2
S4              ds 2
S5              ds 2



;------------------------------------------------------------------------------
; DoAI overlays...

        org Overlay

AI_Vector       ds 2

; SUBSETS of DOAI
; CountDown overlays...

CD_TimerMask    ds 1

; AI_Jump overlays...
AIJ_YSlower     ds 1

; AI_Lurk overlays...
AIL_LurkerCube  ds 1

; CheckMove overlays...

CM_CubeIndex    ds 1
CM_Direction    ds 1
CM_Column       ds 1
CM_scratch      ds 1

; NextLevel overlays...
NL_temp         ds 1

;------------------------------------------------------------------------------
; InitCubes overlays...

        org Overlay

IC_FedCubeIndex ds 1

; SUBSETS OF InitCubes...
; CreateTableaux overlays...

CT_Cubes        ds 1

; FeedCube overlays...

FC_FedCubeIndex ds 1
FC_BlankSpot    ds 1

;------------------------------------------------------------------------------
; CheckJump overlays...

        org Overlay

CJ_CubeIndex    ds 1
CJ_Direction    ds 1
CJ_NewX         ds 1
CJ_NewY         ds 1
CJ_CubeNumber   ds 1
CJ_temp         ds 1

;------------------------------------------------------------------------------
; MoveCube overlay...

        org Overlay

MC_DirOnly      ds 1
MC_temp         ds 1

; SUBSETS OF MoveCube...
; DrawCube overlay...

DC_scratch      ds 1
DC_CubeLine     ds 1
DC_saveX        ds 1

;------------------------------------------------------------------------------

        ; add more overlays here, as required...


    SEG code
    org $1000
; eek! just 4k (-vectors)


;------------------------------------------------------------------------------

SpriteRedirect

        ; The SpriteRedirect table allows the Y-coordinate of a sprite to be
        ; set, and automatically draw the sprite using generic code in the
        ; kernal.  IFF the actual physical line is inside the 0-SPRITE_HEIGHT
        ; range, will the sprite be visible.
        ; Relies on byte #0 of all sprite shapes to be 0

LINE    SET 1
        REPEAT SPRITE_HEIGHT
        .byte LINE
LINE    SET LINE + 1
        REPEND

        REPEAT 256 - SPRITE_HEIGHT -17
        .byte 0                         ; --> guaranteed blank byte in defn.
        REPEND

 ;\--> merges with next table... do not move!
;------------------------------------------------------------------------------

PreDrawn3
 ;---> merges with above 


        REPEAT 17
        .byte 0
        REPEND

    .byte %10000000
    .byte %11000000
    .byte %11100000

;\--> merges with next table... don't move 'em!        

PreDrawn0

    ;     pf1 -->   pf2 <--   pf0 <--   pf1 -->   pf2 <--
    ;     ok        flip      flip xxxx ok        flip
    .byte %11110000     ; 0
    .byte %01111000; 1
    .byte %00111100; 2
    .byte %00011110; 3
    .byte %00001111; 4
    .byte %00000111; 5
    .byte %00000011; 6
    .byte %00000001; 7

PreDrawn4

;\--> merges with next table... don't move 'em!        

        REPEAT 16
        .byte 0
        REPEND

;\--> merges with next table... don't move 'em!        

PreDrawn2

        REPEAT 4
        .byte 0
        REPEND
        
;\--> merges with next table... don't move 'em!        

PreDrawn1

    ;     pf1 -->   pf2 <--   pf0 <--   pf1 -->   pf2 <--
    ;     ok        flip      flip xxxx ok        flip
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000000
    .byte %00000001
    .byte %00000011
    .byte %00000111
    .byte %00001111
    .byte %00011110
    .byte %00111100
    .byte %01111000
    .byte %11110000
    .byte %11100000
    .byte %11000000
    .byte %10000000

        REPEAT 16
        .byte 0
        REPEND

;------------------------------------------------------------------------------

OwnershipBit    .byte %00000001,%00000100,%00010000,%01000000
TargetBit       .byte %00000010,%00001000,%00100000,%10000000
TargetOwnerBit  .byte %00000011,%00001100,%00110000,%11000000

;------------------------------------------------------------------------------

TargetIndex

        ; Points to the correct line in the TargetDisplay playfield
        ; Indexed by the scanline (0-18) being drawn.

        ; This allows the single kernel to draw the entire playfield (!)

        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 0             blank
        .byte TargetDisplay+0,TargetDisplay+0,TargetDisplay+0     ; 1             top row target
        .byte TargetDisplay+0,TargetDisplay+0,TargetDisplay+0     ; 2             top row target
        .byte TargetDisplay+2,TargetDisplay+2,TargetDisplay+2     ; 3             top middle row target
        .byte TargetDisplay+2,TargetDisplay+2,TargetDisplay+2     ; 4             top middle row target
        .byte TargetDisplay+4,TargetDisplay+4,TargetDisplay+4     ; 5             bottom middle row target
        .byte TargetDisplay+4,TargetDisplay+4,TargetDisplay+4     ; 6             bottom middle row target
        .byte TargetDisplay+6,TargetDisplay+6,TargetDisplay+6     ; 7             bottom row target
        .byte TargetDisplay+6,TargetDisplay+6,TargetDisplay+6     ; 8             bottom row target
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 9             blank
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 10            "
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 11            "
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 12            "
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 13            "
        .byte GUARANTEED0READ,GUARANTEED0READ,GUARANTEED0READ     ; 14            "
        .byte TargetDisplay+8,TargetDisplay+8,TargetDisplay+8     ; 15            cube sliding on bottom
        .byte TargetDisplay+8,TargetDisplay+8,TargetDisplay+8     ; 16            "
        .byte TargetDisplay+8,TargetDisplay+8,TargetDisplay+8     ; 17            "
        .byte TargetDisplay+8,TargetDisplay+8,TargetDisplay+8     ; 18            "

;------------------------------------------------------------------------------

bgcol1     .byte    0 ;11000000
pfcol1    .byte    %01111000

DigitNull

        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0
        .byte 0

Digit0
        .byte %00111100
        .byte %01001110
        .byte %10000110
        .byte %10000110
        .byte %10000110
        .byte %10000110
        .byte %01001100
        .byte %00111000

Digit1
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00001100
        .byte %00101100
        .byte %00011100
        .byte %00001100

Digit2
        .byte %11111110
        .byte %11000010
        .byte %01110000
        .byte %00011100
        .byte %00000110
        .byte %00000110
        .byte %10001110
        .byte %01111000

Digit3
        .byte %01111100
        .byte %11000110
        .byte %00000110
        .byte %00000110
        .byte %00011100
        .byte %00000110
        .byte %10000110
        .byte %01111100

Digit4
        .byte %00001100
        .byte %00001100
        .byte %11111110
        .byte %11001100
        .byte %01001100
        .byte %00101100
        .byte %00011100
        .byte %00001100


Digit5
        .byte %01111100
        .byte %11000110
        .byte %00000110
        .byte %00000110
        .byte %00001100
        .byte %01111000
        .byte %01100000
        .byte %01111110

Digit6
        .byte %01111100
        .byte %11000110
        .byte %11000110
        .byte %11000110
        .byte %11111100
        .byte %01100000
        .byte %00110000
        .byte %00001100

Digit7
        .byte %00110000
        .byte %00110000
        .byte %00110000
        .byte %00110000
        .byte %00011000
        .byte %00001100
        .byte %10000110
        .byte %01111110

Digit8
        .byte %01111100
        .byte %11000110
        .byte %11000110
        .byte %11000110
        .byte %01111000
        .byte %01001100
        .byte %01001100
        .byte %00111000

Digit9

        .byte %00011000
        .byte %00001100
        .byte %00000110
        .byte %01111110
        .byte %11100110
        .byte %11000110
        .byte %01000100
        .byte %00111000



        .byte %11111111
        .byte %10000000
        .byte %10110110
        .byte %10110110
        .byte %10000000
        .byte %10110110
        .byte %10110110
        .byte %00000000


        .byte %01101100         ; good player display
        .byte %01001000
        .byte %11111100
        .byte %11000110
        .byte %11100010
        .byte %01011110
        .byte %01110100
        .byte %00111000



;------------------------------------------------------------------------------

Div5

div set 0
        REPEAT 4
        REPEAT CUBEXY
        .byte div
        REPEND
div set div+1
        REPEND

Mod5

        REPEAT 4
div set 0
        REPEAT CUBEXY
        .byte div
div set div+1
        REPEND
        REPEND

;------------------------------------------------------------------------------


        ; Sprite animation is a programmed sequence
        ; Special values (*NOT* available for actual graphics)...

        ; there's colour multiplexing
        ; and there's animation...



SpriteVector

        ; indexed by creature ID/type
        ; points to the sprite shape/definition(s) for each creature

        .word SampleSprite                 ; 0
        .word Watermelon                   ; 1
        .word Pineapple                    ; 2
        .word Octopus                      ; 3

;------------------------------------------------------------------------------


; Sprite Graphics...

        ; SPRITE DATA MUST *NOT* CROSS PAGE BOUNDARY!!

; Format:

        ; byte          meaning
        ; 0             ALWAYS 0        used for blank display
        ; 1             vector (offset) to 2nd shape (can be 0, indicating THIS shape)
        ;               \--> allows multi-colour sprites by using 2 shapes, 2 colours
        ; add the vector if needed, and the offsets from THAT point are...
        ; 0             COLOUR of this frame
        ; ...           frame definitions
        ; ONLY for the 1st shape, 10 '0' bytes, used for rising out of cubes
        ; if the creature doesn't rise, then these bytes are NOT needed
        ; if it does, then they ARE needed for each multiplex colour/shape definition



SampleSprite

        .byte 21                        ; offset to 2nd shape

        .byte %00001100                 ; greenish

        .byte %00000000                 ; The 'rising' backward data
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000                 ; <- effectively blanker byte

        .byte %00111100                 ; actual data
        .byte %01110110
        .byte %01011110
        .byte %11110011
        .byte %11000011
        .byte %10111111
        .byte %11111110
        .byte %01101100
        .byte %01000100
        .byte %01110110

                ; beginning of 2nd multiplex colour

        .byte %00000101                ; greenish

        .byte %00000000                ; The 'rising' backward data
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000                 ; <-- effectively blanker byte

        .byte %00111100
        .byte %01111110
        .byte %01111110
        .byte %11111111
        .byte %11111111
        .byte %11111111
        .byte %11111110
        .byte %01101100
        .byte %01000100
        .byte %01110110

;------------------------------------------------------------------------------

Points20

      ;  .byte %00000000

      ;  .byte %11101110
      ;  .byte %01111011
      ;  .byte %11011011
      ;  .byte %11101110
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000

      ;  .byte %11110110                ; greenish

      ;  .byte %00000000

      ;  .byte %00100010
      ;  .byte %00101001
      ;  .byte %01001001
      ;  .byte %00100010
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000

      ;  .byte %01100000                ; greenish

;------------------------------------------------------------------------------

Octopus
;Octopus - average
 
      ;  .byte %00000000

      ;  .byte %00111100
      ;  .byte %01111110
      ;  .byte %11011011
      ;  .byte %11011011
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %00111100
      ;  .byte %11101110
      ;  .byte %10101011
      ;  .byte %01001001

      ;  .byte %01100110                ; greenish

      ;  .byte %00000000
      ;  .byte %00111100
      ;  .byte %01111110
      ;  .byte %11111111
      ;  .byte %11111111
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %00111100
      ;  .byte %11111110
      ;  .byte %11111111
      ;  .byte %01101011

      ;  .byte %01100010                ; greenish

Pineapple
; BEAUT! Pineapple

      ;  .byte 21
      ;  .byte %00000000

      ;  .byte %01010111
      ;  .byte %10111001
      ;  .byte %00011010
      ;  .byte %00111100
      ;  .byte %01011110
      ;  .byte %01110110
      ;  .byte %01011010
      ;  .byte %01101110
      ;  .byte %01010110
      ;  .byte %00111100

      ;  .byte %11000111                ; purple

      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00000000
      ;  .byte %00111100
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %01111110
      ;  .byte %00111100

      ;  .byte %101000110                 ; red?



Watermelon
; OK watermelon

        .byte 21                        ; offset to 2nd shape

        .byte %101000110                ; red?

        .byte %00000000                 ; The 'rising' backward data
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000                 ; <- effectively blanker byte

        .byte %00000000
        .byte %00000110
        .byte %00011010
        .byte %00110110
        .byte %00011010
        .byte %00111110
        .byte %00101110
        .byte %00011010
        .byte %00000110
        .byte %00000000


        .byte %11000111                ; purple

        .byte %00000000                 ; The 'rising' backward data
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000
        .byte %00000000                 ; <- effectively blanker byte

        .byte %00000110
        .byte %00011000
        .byte %00100000
        .byte %01000000
        .byte %01000000
        .byte %01000000
        .byte %01000000
        .byte %00100000
        .byte %00011000
        .byte %00000110


      

;--------
; start
; program entry point





start
        sei
        cld

    ; clears all variables, stack
    ; init stack pointer
    ; also clears tia registers
    ; does this by "wrapping" the stack - unusual

    ; don't place this in a subroutine - you dunno what the stack is!

        ldx #0
        txs
        pha                     ; best way to get sp=$ff, x=0

        txa
clear   pha
        dex
        bne clear

    ; 9 bytes total for clearing stack, memory
    ; stack pointer now $ff, a=x==0


        ; FOLLOWING IMPLICITLY INITIALISES 0 BY ABOVE...

;        sta SWACNT              ; all input bits



        ;--------
        ; one-time initialization
        ; happens at system reset, no other time

        lda #$6D
        sta rnd
        sta rnd+1


        lda #%01111111
        sta level


        jsr NextLevel

    ; main loop - this is the whole shebang

main    jsr verticalblank       ; start the display
        jsr DrawScreen
        jsr overscan
        jmp main

;--------
; verticalblank

    ; writes the vertical synch signal for the start of a frame
    ; sets up timer so we can keep track of available time - note
    ; there are 37 lines available before the screen needs to start
    ; drawing - approx 2800 cycles therefore available for game
    ; calculations

verticalblank

        lda #D1+D6
        sta VBLANK              ; start with tia disabled


    ; typical frame consists of 3 vertical sync (VSYNC) lines,
    ; 37 vertical blank (vblank) lines,
    ; 192 tv picture lines
    ; 30 overscan lines

    ; ref: nick bensema's wonderful "how to draw a playfield"

    ; require 3 VSYNC lines at start of frame
    ; ref: spg pp.1

        lda #D1
        sta VSYNC               ; start vertical synch

    ; there's 76 cycles available on each of these lines
    ; three required for the VSYNC - don't cut it too close
    ; but do interleave other code here

        sta WSYNC               ; 1st
        sta WSYNC               ; 2nd

    ; we have just 37 lines (@2812 cycles) of vblank, so we have to make
    ; sure the screen draw doesn't get stomped on.  the timer will tell
    ; us if we've screwed up.  timer done just before the last WSYNC to
    ; give maximum possible cycle count availability/accuracy

        lda #44
        sta TIM64T              ; 37.05 scan lines

        lda #0
        sta VSYNC               ; stop vertical synch


    ; screen is now scanning, but tia suppressed
    ; begins again when 0 -> vblank

    ; this is running after the TV VSYNC has been sent, but before
    ; we need to start drawing lines.  there are 37 lines, just around
    ; 2800 cycles.  gotta be sure sure that we don't overflow that time
    ; otherwise we'll get screen jitter

        lda BackColour
        sta COLUBK
        lda pfcol1
        sta COLUPF

                ; << lots of time in here to do stuff
                ; (approx 2800 cycles)


        jsr FeedCube
        jsr CheckTableaux       ; animate target grid
        jsr CheckCompletion


endvb   lda INTIM
        bne endvb

        sta WSYNC
        sta VBLANK              ; end vblank period

        rts

;------------------------------------------------------------------------------


DrawScreen


    ; the kernel
    ; line by line we have to hold hands with the tia, passing it data
    ; so that it draws the screen for us.
    
    ; wait for vertical blank period to end
    ; timer counts down 1 every 64 cycles, so just wait for it
    ; it may already have expired


        lda #%11111100
        sta COLUP0
        sta COLUP1

        lda #227;38
        sta TIM64T                      ; 200.4 scan lines

        sed
        clc
        lda PlayerScore
        adc #$09
        sta PlayerScore
        lda PlayerScore+1
        adc #0
        sta PlayerScore+1
        lda PlayerScore+2
        adc #0
        sta PlayerScore+2
        cld


        sta WSYNC               ; not 100% required


        ;**************************************************
        ; WARNING: VARIABLE USAGE
        ; DO *NOT* PLACE THIS BETWEEN OVERLAY VAR USAGE

        jsr SetupTopScore
        jsr ScoreLine

        ;**************************************************

        jsr PositionSprites

        lda #0 ;D1                 ; reflect / score
        sta CTRLPF
        lda #D0
        sta VDELP0

;------------------------------------------------------------------------------


    ;----
    ; cycle thru drawing line by line...
    ; time is precious here... there are just 228/3 cycles per line
    ; (giving us just 76 cycles) and that's not counting the WSYNC,
    ; should you be writing that. There are 68 pixels (/3 = cycles)
    ; in the hblank period, which starts at the conceptual beginning
    ; of a line on this machine

                ; TOP BORDER
        lda #0
        sta NUSIZ0
        sta NUSIZ1

        jsr CalculateSprites
      
        sta WSYNC

        ldx #0                  ; blank
        REPEAT 1
        jsr MBlock
        REPEND
       


        ldx #3                  ; border
        jsr MBlock
        jsr MBlock
        jsr MBlock
        jsr MBlock

        
        ; Draw the screen with the associated target area display        

                ; INNER TARGET SECTION - 8 "LINES"

        ldx #0

        lda #15
        sta MT_TargetMask       ; mask for border of target area

        jsr MTargetAreaBorder

LINE    SET 0
        REPEAT 8

        jsr MTargetArea         ; drawing a line (with target area)

LINE    SET LINE+1
        REPEND

        jsr MTargetAreaBorder

        REPEAT 5

        jsr MTargetArea         ; drawing a line (with - effectively NO - target )

LINE    SET LINE+1
        REPEND

        lda #255
        sta MT_TargetMask

        REPEAT 4

        jsr MTargetArea         ; drawing the bottom 4 lines with the 'slide-on' cube

LINE    SET LINE+1
        REPEND

        ldx #3                  ; Border
        jsr MBlock
        jsr MBlock
        jsr MBlock
        jsr MBlock


        lda #0
        sta PF0
        sta PF1 
        sta PF2

        sta GRP0
        sta GRP1


        ;**************************************************
        ; WARNING: VARIABLE USAGE
        ; DO *NOT* PLACE THIS BETWEEN OVERLAY VAR USAGE

        jsr SetupBottomScore
        jsr ScoreLine

        ;**************************************************



    ; make sure we have a 192 line display
    ; pal must have an even # or the colours screw up?!

EndOfScreen
        lda INTIM
        bne EndOfScreen

        sta WSYNC

        lda #D6+D1
        sta VBLANK              ; end of screen - enter blanking
    
        rts

;------------------------------------------------------------------------------

overscan

        lda #36                 ; 30+ scan lines of overscan
        sta TIM64T

        dec timer               ; one tick

        jsr CreatureAI
        Jsr MoveCube

ThankES lda INTIM
        bne ThankES

        rts

;------------------------------------------------------------------------------

Adjust
        .byte 1
        .byte 1
        .byte 1
        .byte 1
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        .byte 2
        

AdjustL
        .byte 4
        .byte 3         ; leftmost cube
        .byte 4
        .byte 4
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3
        .byte 3


PositionSprites

        ; Set the horizontal position of the two sprites, based upon their coordinates
        ; This uses the tricky 2600 method of positioning (RESPx, HMPx, etc)

        ; Algorithm invented a looong time ago by persons uknown
        ; Re-invented Feb2001 by yours truly, then optimised according to code by Thomas Jenztsch


        sta HMCLR                       ; clear any previous movement

        ldx #1                          ; sprite index
PosSP   
        lda PlayerX,x
        tay

        lsr
        lsr
        lsr
        lsr
        
        sta PS_temp

        tya
        and #15

        clc
        adc PS_temp

        ldy PS_temp

        cmp #15
        bcc NH
        sbc #15
        iny
NH

                ; Use remainder for fine adjustment
        eor #7
        asl
        asl
        asl
        asl

        sta HMP0,x                      ; fine movement
        sta WSYNC

        jsr Ret
        bit 0                           ; 15 cycles = 3 loops :)

Jiggle  dey
        bpl Jiggle

        sta RESP0,x

        dex
        bpl PosSP

        sta WSYNC
        sta HMOVE

        rts


;------------------------------------------------------------------------------


CheckMove

        ; Determine if it is possible to move in a desired direction
        ; if so, current position is vacated and new position claimed

        ; y = direction (guaranteed 0-3, preserved)
        ; x = cube index (preserve!)

        ; returns carry C = success
        ; S = fail
        ; X,Y preserved

        stx CM_CubeIndex
        sty CM_Direction

         ; apply movement

        lda CubeX,x
        and #31
        cmp #16
        bcs Owned               ; offscreen - no moving
        tax

        lda DirMoveX,y
        adc Div5,x              ; now a horizontal index (0-3)  (RELY ON CARRY C)
        cmp #4                  ; an illegal position?
        bcs Owned               ; can't move there

        sta CM_Column

        ldx CM_CubeIndex
        lda CubeYDir,x
        and #15
        tax

        lda DirMoveY,y
        adc Div5,x              ; (RELY ON CARRY C)
        cmp #4                  ; an illegal position?
        bcs Owned               ; can't move there

        tay                     ; ROW

        ldx CM_Column
        lda Grid,y              ; row of grid
        and OwnershipBit,x
        bne Owned               ; can't go there!


        ; OK, so the grid is available... grab it!

        lda Grid,y
        ora OwnershipBit,x
        sta Grid,y

        ; free the grid currently occupied

        ldx CM_CubeIndex

        lda CubeYDir,x
        and #15
        tay
        lda Div5,y
        tay

        lda CubeX,x
        and #15                 ; GUARANTEED <=15
        tax
        lda Div5,x
        tax

        lda OwnershipBit,x
        eor #$FF
        and Grid,y
        sta Grid,y

                ; Merge new direction into cube information

        lda CM_Direction
        asl
        asl
        asl
        asl
        sta CM_scratch

        ldx CM_CubeIndex
        lda CubeYDir,x
        and #%01001111                  ; keep all but direction bits
        ora CM_scratch
        sta CubeYDir,x

        ldy CM_Direction

        clc
        rts

Owned   ldx CM_CubeIndex
        jsr SetOwner

        ldy CM_Direction

        sec                     ; fail
        rts


;------------------------------------------------------------------------------


CheckJump

        ; pass X = cube index (preserved!)
        ; Y = direction (preserved!)

        ; Tries to jump creature in desired direction

        stx CJ_CubeIndex
        sty CJ_Direction

        cpy #4
        bcc ok4
goJump  jmp NoJump
ok4


        ; apply movement

        lda #0
        cpy #4
        bcs NoMovxx             ; restrict movement to 0-3 directions
        lda DirMoveX,y
NoMovxx

        sta CJ_temp

        lda CubeX,x
        and #31
        cmp #16
        bcs goJump
        tay

        lda CJ_temp
        clc
        adc Div5,y              ; now a horizontal index (0-3)
        cmp #4                  ; an illegal position?
        bcc OKfar
        jmp NoJump              ; can't move there
OKfar


        sta CJ_NewX             ; 0-3 index of X of new position

        ldy CJ_Direction        ; direction to move

        lda #0
        cpy #4
        bcs NoMov2xx
        lda DirMoveY,y
NoMov2xx



        pha

        lda CubeYDir,x
        and #15
        tay

        pla
        clc
        adc Div5,y
        cmp #4                  ; an illegal position?
        bcs goJump              ; can't move there

        tay                     ; 0-3 index of Y of new position


        ldx CJ_NewX             ; 0-3 index of X of new position
        lda Grid,y              ; row of grid
        and OwnershipBit,x
        beq NoJump              ; Can't jump if it's empty!!

                ; Find the cube occupying that grid position
                ; Y = COL
                ; X = ROW (0-3 indexes) = CJ_NewX

        sty CJ_NewY

        ldx #NUMBEROFCUBES-1
FindTarget

        lda CubeX,x
        and #31
        cmp #16
        bcs NextTarget          ; can't jump onto feeding cube
        tay

        lda Div5,y
        cmp CJ_NewX             ; COLUMN?
        bne NextTarget

        lda CubeYDir,x
        and #15
        tay
        lda Div5,y
        cmp CJ_NewY             ; ROW?
        bne NextTarget

                ; FOUND! the target cube index is in x
                ; figure which player is moving

        ldy #0

        lda PlayerCube
        and #7                  ; just cube index
        cmp CJ_CubeIndex        ; cube being checked
        beq IsP0

        lda PlayerCube+1
        and #7                  ; just cube index
        cmp CJ_CubeIndex        ; cube being checked
        bne NoJump

        ldy #1
IsP0
                
        stx CJ_CubeNumber
        
        lda PlayerCube,y
        and #7
        cmp CJ_CubeNumber
        beq JumpOK              ; don't do and do and do
        
        
        lda PlayerCube,y
        and #%11111000
        ora CJ_CubeNumber
        sta PlayerCube,y

        lda #MODE_JUMP + 20
        sta PlayerCounter,y

        lda #15
        sta PlayerY,y

        lda CJ_Direction        ; direction
        bne notup
        lda #-15
        sta PlayerY,y
notup


;        tya
;        tax
;        jsr TieToCube

                ; store the jump direction

        lda PlayerFlags,y
        and #%00111111
        sta PlayerFlags,y

        lda CJ_Direction
        asl
        asl
        asl
        asl
        asl
        asl
        ora PlayerFlags,y
        sta PlayerFlags,y               ; set direction of jump


JumpOK
        ldy CJ_Direction
        ldx CJ_CubeIndex

        clc                     ; success
        rts


NextTarget
        dex
        bpl FindTarget


                ; WHOOPS!
                ; Shouldn't ever get here

                ; fall-through intentional - what else to do!
      

NoJump
        ldy CJ_Direction
        ldx CJ_CubeIndex

        sec                     ; fail
        rts

;------------------------------------------------------------------------------

SetOwner

        ; Markes a grid position as occupied.  Prevents other cubes moving there.
        ; pass X = valid cube index (preserved)

        txa
        pha                             ; use stack, as overlay use is unclear

        lda CubeYDir,x
        and #15
        tay
        lda Div5,y
        tay

        lda CubeX,x
        cmp #16
        bcc XinRange
        lda #15                         ; far-right (ie: feeding-on) cubes are trimmed to ON
XinRange
        tax
        lda Div5,x
        tax

        lda OwnershipBit,x
        ora Grid,y
        sta Grid,y

        pla
        tax
        rts

;------------------------------------------------------------------------------

Random
        ; Generate a pseudo-random number

       lda rnd
       lsr
       lsr
       sbc rnd
       lsr
       ror rnd+1
       ror rnd
       ror rnd
       rts


;------------------------------------------------------------------------------

TieToCube

        ; Horizontally position a sprite to where a cube is
        ; Screwy sprite positioning by the hardware makes this a bit tricky

        ; X = sprite object to tie (0 or 1) (preserved)


        lda PlayerCube,x
        and #7
        tay

        lda CubeX,y
        and #31
        asl
        asl
        adc #SPRITE_X_ADJUST
        sta PlayerX,x

        rts

;------------------------------------------------------------------------------

CreateTableaux

        ; Pass A = number of cubes
        ; randomly enters 'home' position markers into Grid

        sta CT_Cubes

CTab    jsr Random
        
        tay
        and #3
        tax

        tya
        lsr
        lsr
        and #3
        tay

CheckGrid

        lda Grid,x
        and TargetBit,y
        beq EnterPosition

        dey
        bpl CheckGrid
        ldy #3

        dex
        bpl CheckGrid
        ldx #3
        bne CheckGrid

EnterPosition

        lda Grid,x
        ora TargetBit,y
        sta Grid,x

        dec CT_Cubes
        bne CTab

        rts


TargetDisplayBitL

        .byte %00001100
        .byte %00000011

;\--> merges with next table... don't move 'em!        

TargetDisplayBitR

        .byte %00000000
        .byte %00000000
        .byte %00000011
        .byte %00001100


;------------------------------------------------------------------------------

CheckTableaux

        ; Change the TargetDisplay so that cubes flash appropriately
        ; CHECKS/UPDATES ONE SQUARE PER CALL

                ; This next bit is using varied bits of the counter to determine which
                ; of the cubes in the target display is to be checked.  By mangling the
                ; counter, we get the effect not only of a regular (slowish) flash rate,
                ; but we also stagger the order in which things are updated - and thus
                ; get a small but discernable difference between the cubes updates.
                ; This small effect gives some visible border between adjacent cubes
                ; but still allows the entire to flash seemingly simultaneously
    
        lda timer
        lsr
        and #15
        tay
        
        ldx UpdateX,y
        lda UpdateY,y
        tay

        lda Grid,x
        and TargetBit,y
        beq EmptyG              ; no target there

        lda Grid,x
        and TargetOwnerBit,y
        cmp TargetOwnerBit,y
        beq Correct             ; correct square, correct position

        ; So there's a target, but no cube.  It has to flash

        lda timer
        bmi Correct             ; toggle controls flash rate

EmptyG

        txa
        asl
        tax

        ; Turn off any display for this square

        lda TargetDisplayBitL,y
        eor #$FF
        and TargetDisplay,x
        sta TargetDisplay,x

        lda TargetDisplayBitR,y
        eor #$FF
        and TargetDisplay+1,x
        sta TargetDisplay+1,x

        rts

Correct

        txa
        asl
        tax

        ; Turn on any display for this square

        lda TargetDisplayBitL,y
        ora TargetDisplay,x
        sta TargetDisplay,x

        lda TargetDisplayBitR,y
        ora TargetDisplay+1,x
        sta TargetDisplay+1,x

        rts

UpdateX .byte 0,0,1,1,2,2,3,3,0,0,1,1,2,2,3,3
UpdateY .byte 0,2,1,3,0,2,1,3,1,3,0,2,1,3,0,2


;------------------------------------------------------------------------------

CheckCompletion

        ; Check for completion of grid
        ; Switches player to locked when complete


        ldy #3
NextR   ldx #3

NextC   lda Grid,x
        and TargetBit,y
        beq NullSq                      ; no target there

        lda Grid,x
        and TargetOwnerBit,y
        cmp TargetOwnerBit,y
        bne NupNot
        
NullSq  dex
        bpl NextC
        
        dey
        bpl NextR        


        ; Looking good... everything in the right place
        ; but we must wait until all cubes are in their home spots!

        ldx #NUMBEROFCUBES-1
Still   lda CubeYDir,x
        cmp #$FF
        beq StillNo                     ; vacant slot

        and #15
        tay
        lda Mod5,y
        bne NupNot

        lda CubeX,x
        cmp #16
        bcs NupNot
        tay
        lda Mod5,y
        bne NupNot

StillNo dex
        bpl Still

        lda #MODE_LOCK + 7
        sta PlayerCounter               ; lock for completion

NupNot
Ret     rts


;------------------------------------------------------------------------------

MBlock

        ; Draws either a blank scanline (x=0) or a border (x=3)
        ; X unchanged!

     ; LINE 1 -------------------------------

        sta WSYNC               ; 3                             wait for start of hblank

        lda #%00000000          ; 2
        sta PF0                 ; 3 ->8         < (@5)
        sta PF1                 ; 3 ->11        < 28 (@8)
        sta PF2                 ; 3 ->14        < 39 (@11)

        jsr Ret                 ; 12
        jsr Ret                 ; 12
        jsr Ret                 ; 12

        nop                     ; 2
        nop                     ; 2 ->52

        inc SpriteLine          ; 4
        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2
 
        lda (Sprite),y          ; 5[6]
        sta GRP0                ; 3 ->73[74]                    GRAPHICS FOR SPRITE 0


     ; LINE 2 -------------------------------

        sta WSYNC               ; 3

        lda BlockIndex,x        ; 4
        sta PF0                 ; 3
        lda BlockIndex+1,x      ; 4
        sta PF1                 ; 3 ->17        < 28 (@14)
        sta PF2                 ; 3 ->20        < 39 (@17)


        inc SpriteLine+1        ; 5

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2 ->33


    ; right pf

        lda BlockIndex+2,x      ; 4
        sta PF0                 ; 3 ->40        < 49 (@37)

        lda (Sprite+2),y        ; 5[6]

        ldy #0                  ; 2
        sty PF1                 ; 2
        sty PF2                 ; 3 ->52[3]     < 66 (@49[50])

        sta GRP1                ; 3 ->56[57]

        rts


BlockIndex

        ; Determines WHAT is to be displayed on the lines

        .byte %00000000,%00000000,%00000000
        .byte %10000000,%11111111,%11110000


;------------------------------------------------------------------------------



FeedCube

        ; Feed a new cube onto the grid
        ; Count the number of cubes actually IN ACTION.
        ; If there's more to come on, AND the bottom right square is free...
        ; FEED on a cube

        ; returns X = cube index of fed cube (or $FF if none)

        ldx #$FF
        stx FC_FedCubeIndex             ; returned index



        ldy #0                          ; counts cubes-in-action
Feeding inx

        lda CubeYDir,x
        cmp #$FF                        ; uninitialised?
        bne Count

        stx FC_BlankSpot                ; a blank spot
        dey                             ; NOT counted

Count   iny                             ; count cubes
        cpx #NUMBEROFCUBES-1
        bcc Feeding

        ; Y now holds count of cubes which are active
        ; 'FC_BlankSpot' will be $FF if all cubes are active (no index found!)

        cpy Cubes                       ; level # cubes... OK for now.
        bcs NotVacant                   ; already all active!

                ; There's still cubes to come... possibly show a pending cube at right edge
                ; See if the previous cube is fully fed, first...

        ldx #NUMBEROFCUBES-1
FSlider ldy CubeYDir,x
        iny
        beq nValid
        lda CubeX,x
        and #16
        bne NoPend
nValid  dex
        bpl FSlider

        lda #%11110000
        sta TargetDisplay+9             ; show the static 'pending' cube

NoPend  lda Grid+3
        and OwnershipBit+3
        bne NotVacant                   ; can't feed: bottom-right square occupied!


        ; Start cube bottom-right, feeding in to screen

        ldx FC_BlankSpot                ; last blank spot

        lda #31
        sta CubeX,x                     ; Waaaay right
        lda #%00111111
        sta CubeYDir,x                  ; moving left, bottom of screen

        jsr SetOwner

        rts

NotVacant

        ldx FC_FedCubeIndex
        rts

;------------------------------------------------------------------------------


InitCubes

        ldx #7
        lda #16
ClearT  sta TargetDisplay,x
        dex
        bpl ClearT

        lda #0
        sta TargetDisplay+8
        sta TargetDisplay+9  ; SLIDING TARGET AREA

        sta Grid
        sta Grid+1
        sta Grid+2
        sta Grid+3

        lda Cubes
        jsr CreateTableaux

        lda #%0000000
        sta PlayerCube                  ; creature ID #0
        lda #%0001000
        sta PlayerCube+1                ; creature ID #1
 

        ldx #NUMBEROFCUBES-1
        lda #$FF
shiftc  sta CubeYDir,x
        dex
        bpl shiftc

        jsr FeedCube

        stx IC_FedCubeIndex

        lda PlayerCube
        and #%11111000
        ora IC_FedCubeIndex
        sta PlayerCube

        lda PlayerCube+1
        and #%11111000
        ora IC_FedCubeIndex
        sta PlayerCube+1

        ldx #0
        jsr TieToCube                   ; tie player to it's cube
        ldx #1
        jsr TieToCube

        rts

;------------------------------------------------------------------------------

NextLevel

        lda bgcol1
        sta BackColour

        lda level
        and #%11111

        clc
        adc #1
        cmp #30
        beq Unchange            ; maxed out

        sta NL_temp

        lda level
        and #%11100000
        ora NL_temp
        sta level

Unchange

        lda level
        and #%11111
        clc
        adc #3
        cmp #NUMBEROFCUBES
        bcc LimitC
        lda #NUMBEROFCUBES
LimitC  sta Cubes

        jsr InitCubes

        ldx #0
        jsr InitCreature
        ldx #1
        jsr InitCreature

        rts

;------------------------------------------------------------------------------

DrawCube

        ; Draw/Erase a single cube at x,y location

        ; pass
        ; X     = index of cube (preserved)
        ;       CUBE MUST EXIST (That is, no passing empty slots)

        ; the y position simply gives us the ROW (*5) of the start of draw
        ; the X position is mangled to give us byte information.
        ; TRICKY due to the odd layout of playfields

        ; use indexing into (predrawn) table lines to make it simple
        ; 5 bytes per x position = 

        stx DC_saveX

        ldy CubeX,x
        cpy #16
        bcc NoBottom

                ; Draw the sliding cube on the bottom

        lda PreDrawn3,y
        sta TargetDisplay+8
        lda PreDrawn4,y
        sta TargetDisplay+9            ; this is pretty neato... just two bytes for the bottom sliding cube :))

NoBottom

                ; Draw in the grid area playfield

        lda CubeYDir,x
        and #15
        sta DC_scratch
        asl
        adc DC_scratch
        tax                             ; line x 3 -> playfield location of start

        lda #3
        sta DC_CubeLine


DrawLoop

        lda PreDrawn0,y
        ora playfield,x
        sta playfield,x

        lda PreDrawn1,y
        ora playfield+1,x
        sta playfield+1,x

        lda PreDrawn2,y
        ora playfield+2,x
        sta playfield+2,x

        inx
        inx
        inx

        dec DC_CubeLine
        bpl DrawLoop

        ldx DC_saveX
        rts

;------------------------------------------------------------------------------

; Place table before code (for DASM, avoiding "Label Mismatch" warnings)

joystickdir
    .byte $FF
    .byte 0     ; up
    .byte 2     ; down
    .byte $FF,3  ; left
    .byte $FF,$FF,$FF,1       ; right
    .byte $FF,$FF,$FF,$FF,$FF,$FF,$FF


Joystick
 
        ; X unchanged (preserved)
        ; pass X= cube index

        ; returns
        ; A = direction (0-3) or $FF (stopped)

;        lda #0
;        sta SWACNT              ; all input bits (IMPLICITLY SET @ STARTUP)

        lda SWCHA               ; joystick
        eor #$FF
        lsr
        lsr
        lsr
        lsr
        and #15
        tay

        lda joystickdir,y
        rts
    


;------------------------------------------------------------------------------

InitCreature

        ; Start a creature.  It will slide up out of the cube
        ; pass X = creature index (0 or 1)

        lda #MODE_RISE + SPRITE_HEIGHT-1
        sta PlayerCounter,x

        lda #FLAG_RISE
        sta PlayerFlags,x

        lda #0
        sta PlayerY,x                   ; no vertical offset

        rts

;------------------------------------------------------------------------------

CreatureAI

        ; Perform AI/logic for creatures
 
        ldx #0
        jsr DoAI
        ldx #1
        jsr DoAI

        rts


DoAI

        ; x = creature index

        lda PlayerCounter,x

        lsr
        lsr
        lsr
        lsr

        and #%1110
        tay

        lda AIVector,y
        sta AI_Vector
        lda AIVector+1,y
        sta AI_Vector+1
        jmp (AI_Vector)

AIVector

        .word AINormal
        .word AIRiseUp
        .word AISink
        .word AILurk
        .word AIJump
        .word AILock
        .word AIDelay
        ; etc..


;------------------------------------------------------------------------------

AIJump

        ; Creature is mid-jump
        ; The counter gives us the index into the jump table
        ; The entry at the table gives us the height (y offset) for this frame

 
        lda timer
        and #%11
        bne Jumping

        lda PlayerCounter,x
        and #%11111
        beq EndJump

        dec PlayerCounter,x
        tay
        dey

        and #3
        sta AIJ_YSlower

        sec
        lda PlayerY,x
        sbc JumpVector,y
        sta PlayerY,x

        lda PlayerFlags,x
        asl
        rol
        rol
        and #3
        tay
        
        clc
        lda PlayerX,x
        adc DirMoveX,y
        sta PlayerX,x
         
        lda AIJ_YSlower
        beq nY
            
        sec
        lda PlayerY,x
        sbc DirMoveY,y
        sta PlayerY,x

nY

        
Jumping

        
        rts

EndJump

        lda #MODE_DELAY + TIME_PAUSE_AFTER_JUMP
        sta PlayerCounter,x

        lda #0
        sta PlayerY,x

        rts


HMotion
        .byte 0
        .byte %11110000
        .byte 0
        .byte %00010000


;------------------------------------------------------------------------------

JumpVector

        ; Has to be 20 long.  It's a long story.

 .byte 2,2,2,1,1,1,1,0,1,0
 .byte 0,-1,0,-1,-1,-1,-1,-2,-2,-2
 
 ;.byte 0,2,3,4,5,6,6,7,8,9,9
 ;.byte 9,9,8,7,6,6,5,4,3,2

        ; Jumps over 20 pixels

 ;       .byte 0                                 ; NON-jump




;------------------------------------------------------------------------------

AIDelay

        lda #%1111
        jsr CountDown
        bcc ToNormal
        rts

;------------------------------------------------------------------------------

CountDown

        ; Generic decrement of the player's counter
        ; pass x = player, a = timer mask
        ; returns carry S = counter still active (non-zero on entry)
        ; C = counter halted (was 0)

        and timer
        bne NotEnd

        lda PlayerCounter,x
        and #%11111
        beq Go0

        dec PlayerCounter,x

NotEnd  sec
        rts
        
Go0     clc
        rts



;------------------------------------------------------------------------------

AILock

        ; Locked, pending end of screen

        lda #%1111
        jsr CountDown
        bcc ToEnd
        
        rts


ToEnd
        jsr NextLevel
        rts

;------------------------------------------------------------------------------


AINormal

        ; Currently just sits around waiting to sink

        cpx #0
        beq Human

;        jsr Random
;        cmp #3
;        bcs Normaling
        jsr Random
        cmp #20
;        bcs Normaling


        lda #MODE_SINK + 0
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x               ; use counter as rise/fall value

Normaling

        rts



;------------------------------------------------------------------------------

Human

        ; Object is human-controlled

        jsr Joystick
        cmp #4
        bcs UnJoy                       ; no legal joystick direction
        tay


        lda PlayerCube,x
        and #7
        tax

        lda CubeYDir,x
        bpl UnJoy                       ; Can't do anything if cube in motion

        jsr CheckMove        
        bcc UnJoy                       ; OK!

         ; Couldn't move the cube...
        ; MIGHT be able to 'jump' to the adjacent cube

        jsr CheckJump

UnJoy   rts

;------------------------------------------------------------------------------





AIRiseUp

        ; Rise up out of a cube

        lda #%11111
        jsr CountDown
        bcc ToNormal
  
        rts


ToNormal

        lda #MODE_NORMAL
        sta PlayerCounter,x             ; switch to normal AI

        lda PlayerFlags,x
        and #255-FLAG_RISE
        sta PlayerFlags,x

        lda #0
        sta PlayerY,x

        jsr TieToCube
        rts


AISink


        lda PlayerCounter,x
        and #%11111
        cmp #SPRITE_HEIGHT-2 ;???
        bcs Repos

        lda timer
        and #%111111
        bne Sinker

        inc PlayerCounter,x

Sinker
        rts

Repos



        ; Lurking, waiting for a cube

        lda #MODE_LURK + SPRITE_HEIGHT-1
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x

        rts


AILurk


        ; choose a new cube

        jsr Random
        cmp #30
 ;       bcs Lurker
        jsr Random
        cmp #220
 ;       bcc Lurker

 
        and #7
        cmp #NUMBEROFCUBES
        bcs Lurker

 
        tay
 
        lda CubeYDir,y
        cmp #$FF
        beq Lurker                      ; cube not present

        sty AIL_LurkerCube

        lda PlayerCube,x
        and #%11111000
        ora AIL_LurkerCube
        sta PlayerCube,x

        jsr TieToCube

        lda #MODE_RISE + SPRITE_HEIGHT-1
        sta PlayerCounter,x

        lda PlayerFlags,x
        ora #FLAG_RISE
        sta PlayerFlags,x

Lurker
        rts



;------------------------------------------------------------------------------

MoveCube


        lda timer
        and #31
        beq toofar
        jmp noMove                      ; limit speed of cube movement
toofar
        
        ; Erase all in a single swipe

        ldy #PFSIZE-3
        lda #0
        ldx #128
ZapPF   sta playfield,y
        sta playfield+1,y
        stx playfield+2,y
        dey
        dey
        dey
        bpl ZapPF

        ; Move cubes and draw

        ldx #NUMBEROFCUBES-1
ShiftC2

        lda CubeYDir,x
        cmp #$FF
        beq Incomplete                  ; non-cube

        lda CubeYDir,x
        bmi stopped

        lsr
        lsr
        lsr
        lsr
        and #3
        tay                     ; 0-3 direction

        lda CubeX,x
        and #%11100000
        sta MC_temp
         
        clc
        lda CubeX,x
        adc DirMoveX,y
        and #31
        ora MC_temp
        sta CubeX,x

        lda CubeYDir,x
        and #$f0
        sta MC_DirOnly

        clc
        lda CubeYDir,x
        adc DirMoveY,y
        and #$0F
        ora MC_DirOnly
        sta CubeYDir,x


                ; Shift ALL players on this cube

        txa
        eor PlayerCube
        and #7
        bne not0

        lda DirMoveX,y
        asl
        asl
        clc
        adc PlayerX
        sta PlayerX

not0

        txa
        eor PlayerCube+1
        and #7
        bne not1

        lda DirMoveX,y
        asl
        asl
        clc
        adc PlayerX+1
        sta PlayerX+1

not1



stopped 

        jsr DrawCube                    ; DRAW call cubes

        ldy CubeX,x
        cpy #16
        bcs Incomplete

        lda Mod5,y
        bne Incomplete

        lda CubeYDir,x
        and #15
        tay
        lda Mod5,y
        bne Incomplete

                ; hit a gridlock boundary, so halt

        lda CubeYDir,x
        and #15
        ora #$80
        sta CubeYDir,x                  ; halt movement

Incomplete
        
        dex
        bmi noMove
        jmp ShiftC2

noMove
        rts

DirMoveY        .byte $fF       ;...continues...
DirMoveX        .byte 0,$01,0,$FF


MTargetArea

     ; LINE 1 -------------------------------

        sta WSYNC               ; 3                             wait for start of hblank

        lda #%00000000          ; 2
        sta PF0                 ; 3 ->8         < (@5)
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->15        < 28 (@12)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->22        < 39 (@19)

        inc SpriteLine          ; 4

    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->35        < 49 (@32)

        ldy TargetIndex,x       ; 3
        lda 0,y                 ; 4
        sta MTA_TargetL         ; 3
        and MT_TargetMask       ; 3
        sta PF1                 ; 3 ->51        < 55 (@48)
        lda 1,y                 ; 4
        sta MTA_TargetR         ; 3
        and MT_TargetMask       ; 3
        sta PF2                 ; 3 ->64        < 66 (@61)

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2
 
        lda (Sprite),y          ; 5[6]
        sta GRP0                ; 3 ->81[82] (5[6] OVER)               GRAPHICS FOR SPRITE 0


     ; LINE 2 -------------------------------

        ; OVERSCANNED BY 5[6] CYCLES (OK)

        lda #%10000000          ; 2 ->8
        sta PF0                 ; 3
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->18        < 28 (@15)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->25        < 39 (@22)

        inc SpriteLine+1        ; 4

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->45        < 49 (@42)

        lda MTA_TargetL         ; 3
        sta PF1                 ; 3 ->51        < 55 (@48)
        lda MTA_TargetR         ; 3
        sta PF2                 ; 3 ->57        < 66 (@54)

        inc SpriteLine          ; 4

        lda (Sprite+2),y        ; 5[6]
        sta GRP1                ; 3 ->69[70]

        lda #%00000000          ; 2
        ldy playfield,x         ; 4 ->75[76]


     ; LINE 3 -------------------------------

        sta PF0                 ; 3 ->3         < (@0)
        sty PF1                 ; 3 ->6         < 28 (@3)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->13        < 39 (@10)

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

 nop ;2
 nop ;2
 nop ;2
 bit 0 ;3


    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->40        < 49 (@37)

        lda MTA_TargetL         ; 3
        and MT_TargetMask       ; 3
        sta PF1                 ; 3 ->49        < 55 (@46)
        lda MTA_TargetR         ; 3
        and MT_TargetMask       ; 3
        sta PF2                 ; 3 ->58        < 66 (@55)

        lda (Sprite),y          ; 5[6]

 bit 0 ;3

        sta GRP0                ; 3 ->69[70]                    GRAPHICS FOR SPRITE 0

 
        lda #%10000000          ; 2
        ldy playfield,x         ; 4 ->75[6]
 
     ; LINE 4 -------------------------------

        sta PF0                 ; 3
        sty PF1                 ; 3 ->6         < 28 (@3)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->13        < 39 (@10)

        inc SpriteLine+1        ; 4

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2
 
 nop ;2
 nop ;2
 nop ;2
 bit 0 ;3

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->42        < 49 (@39)

        lda MTA_TargetL         ; 3
        sta PF1                 ; 3 ->48        < 55 (@45)
        lda MTA_TargetR         ; 3
        sta PF2                 ; 3 ->54        < 66 (@51)

        lda (Sprite+2),y        ; 5[6]
        sta GRP1                ; 3 ->62[63]

 nop ;2
 nop ;2
 bit 0 ;3

        inc SpriteLine+1        ; 5


        lda #%00000000          ; 2 ->75[6]

     ; LINE 5 -------------------------------

        ldy playfield,x         ; 4

        sta PF0                 ; 3 ->7         < (@3)
        sty PF1                 ; 3 ->10        < 28 (@7)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->17        < 39 (@14)

        inc SpriteLine          ; 4

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->39        < 49 (@36)

        lda MTA_TargetL         ; 3
        and MT_TargetMask       ; 3
        sta PF1                 ; 3 ->48        < 55 (@45)
        lda MTA_TargetR         ; 3
        and MT_TargetMask       ; 3
        sta PF2                 ; 3 ->57        < 66 (@54)

        lda (Sprite),y          ; 5[6]
        sta GRP0                ; 3 ->65[6]                     GRAPHICS FOR SPRITE 0

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2 ->74[5]

 nop ;2 ->76[7]

     ; LINE 6 -------------------------------

                ; OVER BY 1 (OK)


        lda #%10000000          ; 2 ->3
        sta PF0                 ; 3
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->13        < 28 (@10)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->20        < 39 (@17)

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->27        < 49 (@24)

        inx                     ; 2
        inx                     ; 2
        inx                     ; 2

 nop ;2
 nop ;2

        lda MTA_TargetL         ; 3
        sta PF1                 ; 3 ->43        < 55 (@40)
        lda MTA_TargetR         ; 3
        sta PF2                 ; 3 ->49        < 66 (@46)

        lda (Sprite+2),y        ; 5[6]
        sta GRP1                ; 3 ->57[8]

        rts


;------------------------------------------------------------------------------

MTargetAreaBorder

        ; TIMING COMMENTS NOT CHECKED


     ; LINE 1 -------------------------------

        sta WSYNC               ; 3                             wait for start of hblank

        lda #%00000000          ; 2
        sta PF0                 ; 3 ->8         < (@5)
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->15        < 28 (@12)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->22        < 39 (@19)

        inc SpriteLine          ; 4

    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->35        < 49 (@32)

        ldy TargetIndex,x       ; 3
        lda 0,y    ; 4
        lda #0                 ; 2
 cmp 0 ; 3
        sta PF1                 ; 3 ->50        < 55 (@47)
        lda TARGETSIZE,y    ; 4
        lda #0                 ; 2
 cmp 0 ; 3
        sta PF2                 ; 3 ->62        < 66 (@59)

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2
 
        lda (Sprite),y          ; 5[6]
        sta GRP0                ; 3 ->79[80] (3[4] OVER)               GRAPHICS FOR SPRITE 0


     ; LINE 2 -------------------------------

        ; OVERSCANNED BY 3[4] CYCLES (OK)

        lda #%10000000          ; 2 ->6
        sta PF0                 ; 3
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->16        < 28 (@13)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->23        < 39 (@20)

        inc SpriteLine+1        ; 5

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->43        < 49 (@40)

 nop
        lda #31 ;temp                ; 3
        sta PF1                 ; 3 ->49        < 55 (@46)
        lda #31 ;temp+1              ; 3
        sta PF2                 ; 3 ->55        < 66 (@52)

        inc SpriteLine          ; 4

        lda (Sprite+2),y        ; 5[6]
 nop ;2
        sta GRP1                ; 3 ->67[8]

        lda #%00000000          ; 2
        ldy playfield,x         ; 4 ->74


     ; LINE 1 -------------------------------

        sta PF0                 ; 3 ->3         < (@0)
        sty PF1                 ; 3 ->6         < 28 (@3)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->13        < 39 (@10)

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

 nop ;2
 nop ;2
 nop ;2
 bit 0 ;3


    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->40        < 49 (@37)

        lda #0 ; temp           ; 2
 bit 0 ; 3
        sta PF1                 ; 3 ->48        < 55 (@45)

 nop ;2
 bit 0 ;3

        sta PF2                 ; 3 ->56        < 66 (@53)

        lda (Sprite),y          ; 5[6]

 bit 0 ;3
 nop ;2

        sta GRP0                ; 3 ->69[70]                    GRAPHICS FOR SPRITE 0

 
        lda #%10000000          ; 2
        ldy playfield,x         ; 4 ->75[76]
 
     ; LINE 2 -------------------------------

        sta PF0                 ; 3
        sty PF1                 ; 3 ->6        < 28 (@3)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->13        < 39 (@10)

        inc SpriteLine+1        ; 5

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2
 
 nop ;2
 nop ;2
 nop ;2
 bit 0 ;3

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->42        < 49 (@39)
 nop ;2
        lda #31                 ; 3
        sta PF1                 ; 3 ->48        < 55 (@45)
        lda #31                 ; 3
        sta PF2                 ; 3 ->54        < 66 (@51)

        lda (Sprite+2),y        ; 5[6]
        sta GRP1                ; 3 ->62[63]

 nop ;2
 nop ;2
 bit 0 ;3

        inc SpriteLine+1        ; 5

        lda #%00000000          ; 2 ->75[76]

     ; LINE 1 -------------------------------

        ldy playfield,x         ; 4

        sta PF0                 ; 3 ->7         < (@3)
        sty PF1                 ; 3 ->10        < 28 (@7)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->17        < 39 (@14)

        inc SpriteLine          ; 4

        ldy SpriteLine          ; 3                             THE FIRST SPRITE INDEX
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

    ; right pf

        lda playfield+2,x       ; 4
        and #$7f                ; 2
        sta PF0                 ; 3 ->39        < 49 (@36)

 bit 0 ; 3
        lda #0                  ; 2
        sta PF1                 ; 3 ->47        < 55 (@44)
 bit 0 ; 3
 nop ;2
        sta PF2                 ; 3 ->55        < 66 (@52)


        lda (Sprite),y          ; 5[6]
        sta GRP0                ; 3 ->63[64]                    GRAPHICS FOR SPRITE 0

        ldy SpriteLine+1        ; 3                             GRAPHICS FOR SPRITE 1
        lda SpriteRedirect,y    ; 4
        tay                     ; 2

 bit 0 ;3 ->75[76]

     ; LINE 2 -------------------------------

        lda #%10000000          ; 2
        sta PF0                 ; 3 -> 5
        lda playfield,x         ; 4
        sta PF1                 ; 3 ->12        < 28 (@9)
        lda playfield+1,x       ; 4
        sta PF2                 ; 3 ->19        < 39 (@16)

    ; right pf

        lda playfield+2,x       ; 4
        sta PF0                 ; 3 ->26        < 49 (@23)

        inx                     ; 2
        inx                     ; 2
        inx                     ; 2

 nop ; 2
 nop ; 2
 nop ; 2
 nop ; 2
 nop ; 2
 nop ; 2

        lda #31 ;temp                ; 3
        sta PF1                 ; 3 ->50        < 55 (@47)
        lda #31 ;temp+1              ; 3
        sta PF2                 ; 3 ->56        < 66 (@53)

        lda (Sprite+2),y        ; 5[6]
        sta GRP1                ; 3 ->64[65]

        rts

;******************************************************

CalculateSprites

        ; Setup vars for graphics draw of sprites
        ; HORRIBLY INEFFICIENT-LOOKING


        ldx #1

SetupSprite

        lda PlayerCube,x
        lsr
        lsr
        and #14
        tay

        lda SpriteVector,y
        sta DS_Sprite
        lda SpriteVector+1,y
        sta DS_Sprite+1                   ; --> vector to p2

        ldy #0

        lda timer
        and #1
        beq Zerooo7

        lda (DS_Sprite),y                 ; offset to sprite 1 (=0 for single-colour sprites)
        tay

Zerooo7 iny
        lda (DS_Sprite),y
        sta COLUP0,x                      

                ; Now adjust for the actual 'shift' value

        sty DS_temp

        tya
        clc
        adc #SPRITE_HEIGHT
        tay

        lda PlayerFlags,x
        and #FLAG_RISE
        beq NotRisingx7

;        sec
        lda #SPRITE_HEIGHT+1
        sbc PlayerCounter,x
        and #%11111

        clc
        adc DS_temp
        tay

NotRisingx7

        sty DS_temp

        txa
        asl
        tay

;        sec
        lda DS_temp
        adc DS_Sprite
        sta Sprite,y
        lda DS_Sprite+1
        adc #0
        sta Sprite+1,y

        lda PlayerY,x
        lsr
        sta DS_HalfY

        lda PlayerCube,x
        and #7
        tay                             ; just cube index
        lda CubeYDir,y
        and #15                         ; cube Y index

        sta DS_temp
        asl
        adc DS_temp
        sbc DS_HalfY
        eor #$FF
        sbc #SPRITE_Y_ADJUST
        sta SpriteLine,x                ; line offset (neg) for start of sprite draw

        dex
        bpl SetupSprite

        rts

;------------------------------------------------------------------------------


SetupBottomScore

        ; Display lives/level if joystick button pressed


        ldy #10
        lda #0
ClSC    sta S0,y
        dey
        dey
        bpl ClSC


        lda #11
        sta S0

        ldx #0
        sec
        lda level
        and #%11111
        adc #0
        sec
m10     inx
        sbc #10
        bcs m10
        stx S1
        adc #11
        sta S2

        lda #12
        sta S4
        lda level
        lsr
        lsr
        lsr
        lsr
        lsr

        clc
        adc #1
        sta S5

        sta WSYNC
        rts


SetupTopScore

        ; Place appropriate data in score display line

        ldx #2
        ldy #0
inscore
        
        lda PlayerScore,x
        and #15
        sta S0+2,y

        lda PlayerScore,x
        lsr
        lsr
        lsr
        lsr
        sta S0,y

        iny
        iny
        iny
        iny

        dex
        bpl inscore

 
                 ; Now remove leading zeros

        lda #0
        tax
        ldy #1
rlz     ora S0,x
        beq nlz
        inc S0,x
nlz     inx
        inx
        cpx #10
        bcc rlz

        rts

;------------------------------------------------------------------------------

        org $1F00

ScoreLine

        ; pass the 6 'digits' in S0-S5 inclusive
        ; mangled on return


        tsx
        stx stkp

        ldy #0
        sty VDELP0

initscore
        
        lda S0,y
        asl
        asl
        asl
        adc #<DigitNull
        sta S0,y
        lda #>DigitNull
        sta S0+1,y

        iny
        iny

        cpy #12
        bcc initscore
 

        lda #D0                 ; reflect / score
        sta CTRLPF

        ; WARNING:
        ; DIGIT DATA CANNOT CROSS PAGE BOUNDARY!

         lda #255 ;%11100110                 ; greenish

         sta COLUP0
      ;   lda #%10001110 ;blue
         sta COLUP1

         lda #3
         sta NUSIZ0
         sta NUSIZ1 ;3 copies close
         sta WSYNC


        ldx #22
blah2   dex
        bne blah2
        sta RESP0


        sta WSYNC
        ldx #22
blah1   dex
        bne blah1
        nop
        sta RESP1
           
        sta HMCLR
        lda #%10100000;2 more than belo
        sta HMP1
        lda #%11000000;
        sta HMP0
        sta WSYNC
        sta HMOVE


         ;make pointers to num data
        ldx #5
        ldy #0
loop2x  cmp PlayerScore,x
        asl 
        asl 
        asl 

        adc #<DigitNull
        cmp S0,y
        lda #0
        adc #>DigitNull
        cmp S0+1,y
        iny
        iny

        dex
        bpl loop2x


        nop                     ; shift colour change along a bit
        lda #0
        sta PF2
        lda #10
        sta WSYNC
;        sta COLUBK
        sta WSYNC

        ;draw score
        lda #7 ;*2
        sta loop

        ldx #10
pause1  dex
        bne pause1
        nop
        nop
;        nop


        ; Tomas Jentzsch magic...

        ldy #7
mainloop

        sty loop                ; 3
;        lsr                     ; 2
;        tay                     ; 2

 nop
 nop

        lda (S0),y ;5
        sta GRP0                ; 3 = 15

 bit 0 ;       sty COLUBK

        lda (S1),y              ; 5
        sta GRP1                ; 3
        .byte $b3,S5            ; 5 lax (ptr5),y ;5        illegal opcode
        txs                     ; 2
        lda (S2),y              ; 5
        sta sreg                ; 3
        .byte $b3,S3            ; 5 lax (ptr3),y ;5        illegal opcode
        lda (S4),y              ; 5 = 33

        ldy sreg                ; 3
        sty GRP0                ; 3
        stx GRP1                ; 3
        sta GRP0                ; 3
        tsx                     ; 2
        stx GRP1                ; 3 = 17

;        dec loop                ; 5
;        bpl mainloop            ; 3 =  8


        lda loop                ; 3
        tay
        eor #$FF
        sta COLUP0
        sta COLUP1
          nop


        lda (S0),y ;5
        sta GRP0                ; 3 = 15

  bit 0;      sty COLUBK

        lda (S1),y              ; 5
        sta GRP1                ; 3
        .byte $b3,S5            ; 5 lax (ptr5),y ;5        illegal opcode
        txs                     ; 2
        lda (S2),y              ; 5
        sta sreg                ; 3
        .byte $b3,S3            ; 5 lax (ptr3),y ;5        illegal opcode
        lda (S4),y              ; 5 = 33

        ldy sreg                ; 3
        sty GRP0                ; 3
        stx GRP1                ; 3
        sta GRP0                ; 3
        tsx                     ; 2
        stx GRP1                ; 3 = 17

        ldy loop
        dey
;        dec loop                ; 5
        bpl mainloop            ; 3 =  8


        lda #0
        sta PF2

        ldx stkp
        txs

        lda #0
        sta GRP0
        sta GRP1

        rts


;------------------------------------------------------------------------------


    org $1FFC
    .word start                 ; irq
    .word start                 ; reset vector

;EOF
